
//////////////////////////////////////////////////////////////////////
//
// File: gate.c
//
// Author: J F Main.
//
// Description:
//
//   Routines moved here as they are timing critical and
//
//   Only code used by the gate_loop is required here.
//
// Version: 1.00 - Initial release.
//
// Copyright : Copyright  John Main 2006
//   http://www.best-microcontroller-projects.com
//   Free for non commercial use as long as this entire copyright notice
//   is included in source code and any other documentation.
//
//////////////////////////////////////////////////////////////////////
#include "def.h"
#include "bit.h"
#include "gate.h"
#include "delay.h"

// Externally declared variables
extern volatile unsigned short sdmy;
extern volatile unsigned int   udmy;

// Variables local to this file
extern volatile unsigned int st_TMR1O;

////////////////////////////////////////////////////////////////////////
//
// gate_loop_action
//
// This routine contains the action that is to be performed during the
// gate loop - and this action must take constant time so that accurate
// time delays can be measured for the frequency counter.
//
// In this case it is the seven segment display update which
// must also take constant execution time.
//
// Note:
//   All routines called and code here must have constant execution time.
//
// org is used to keep the routine at the same location
// in ROM so that its timing does not change due to page
// boundaries - when change the code.
// org 255
void gate_loop_action(char *segDispStr)  {
unsigned short test;


   display_str_8seg7(&segDispStr[2],20);  // 10 digits from long2str 1 per call.

   // check for overflow
   sdmy = PIR1 & (1<<TMR1IF);

   #ifdef PATH2
      asm { CLRW }
   #endif

   //test = ~test;  // debug - use debugger to set test then tune.
   if ( sdmy!=0 ) {
      PIR1      &= ~(1<<TMR1IF) ; // clear TMR1 overflow bit.
      st_TMR1O++;                 // increase high count.

   } else {
      sdmy &= ~(1<<TMR1IF); ;
      udmy += 1;
      asm {
         CLRW
      }
   }
}

////////////////////////////////////////////////////////////////////////
//
// gate_loop
//
// Note:
//
//   Tune the loop to 999us (just less than 1 millisecond).
//
//   This lets the caller make the time exact by adding delay.
//
//   Routine: gate_loop_action - the user action routine must
//    have constant execution time.
//
//   Delay of gate_loop_action and delay_us must come to
//     less than 1ms so can correct in calling routine.
//
//  TUNE this loop by adding / removing delays and
//       CLRW statements - 1 machine cycle.
//
// Arguments:
//  gt          - gate time in approximate ms.
//  segDispStr  - string to display on seven segment.
//
// Tuning - break on 'gt--;'
// start timer
// simulate until land on 'gt--;' again - count should be 999us.
//
void gate_loop(unsigned int gt, char *segDispStr) org 340 {

   do {

      gt--;

      gate_loop_action(segDispStr);  // user defined action.

      delay_500_us();
      delay_100_us();
      delay_100_us();
      delay_100_us();
      delay_10_us();
      delay_10_us();
      delay_10_us();
      delay_10_us();
      delay_10_us();
      delay_10_us();

      asm {
         CLRW
         CLRW
         CLRW
         CLRW
         CLRW
         
         CLRW
         CLRW
         CLRW
         CLRW
      }

   } while (gt>0);
}





