// File usart.c

#include "bit.h"

//////////////////////////////////////////////////////////////////////
void USART_Print(char *str) {
   while (*str) {
      USART_Write(*str++);
      delay_ms(5); // should be able to poll h/w
      // Note takes 4.16ms to Tx a char at 2400 baud
   }
}

//////////////////////////////////////////////////////////////////////
void USART_Println(char *str) {

   while (*str) {  // why not use USART_Print - uses the call stack
      USART_Write(*str++);
      delay_ms(5);
   }
   USART_Write('\r');
   delay_ms(5);
   USART_Write('\n');
   delay_ms(5);
}

//////////////////////////////////////////////////////////////////////
/*
void Soft_Uart_Println(char *str) {
   while (*str) {  // why not use USART_Print - uses the call stack
       Soft_Uart_Write(*str++);
   }
   Soft_Uart_Write('\r');
   Soft_Uart_Write('\n');
}

//////////////////////////////////////////////////////////////////////
void Soft_Uart_Print(char *str) {
   while (*str) {  // why not use USART_Print - uses the call stack
      Soft_Uart_Write(*str++);
   }
}
*/

void _UART_Init(void) {
  RCSTA = (1<<SPEN);
}

//////////////////////////////////////////////////////////////////////
// Write a character to a pin as RS232 serial data
//
// Baud 2400
//
// 1 start bit
// 8 data bits
// No parity
// 1 stop bit
//
//////////////////////////////////////////////////////////////////////
// Marcos that define ports for the uart
// Faster this way so more likely to work.
//
#define SU_set setBit(PORTB,4)
#define SU_res resBit(PORTB,4)
//
//
void _Soft_Uart_Write(unsigned short chr) {
unsigned short mask=1,i;
unsigned int data;

   data = chr << 1;
   data &= ~0x0001; // START bit (=0).

   // 10 output bits
   for (i=0;i<9;i++) { // lsb 1st  8 bits
      if (mask & data) {
         SU_set; // output low for logic 1
      } else {
         SU_res;// output high for logic 0
      }
      mask <<=1;
      delay_us(364);  // simulate to calibrate the loop.
   }

   SU_set; // stop bit
   delay_us(417);
}



////////////////////////////////////////////////////////////////////////
//// works
////  Write a character to a pin as RS232 serial data
//    // 1 start bit
//    // 8 data bits
//    // No parity
//    // 1 stop bit
////
//void _Soft_Uart_Write(unsigned short chr) {
//unsigned short mask=1,i;
//volatile unsigned ch = 0;
//unsigned int out[10];
//unsigned int data;
//
//
//data = chr << 1;
//data &= ~0x0001; // START bit
////data |= 0x200;   // Stop bit
//
//setBit(PORTA,2);
//
//   // 10 output bits
//   for (i=0;i<9;i++) { // lsb 1st  8 bits
//         if (mask & data) {
//            setBit(PORTB,4); // output low for logic 1
//         } else {
//            resBit(PORTB,4); // output high for logic 0
//         }
//         mask <<=1;
//      delay_us(364);
//   }
//
//setBit(PORTB,4); // stop bit
//delay_us(417);
//
//resBit(PORTA,2);
//}

//////////////////////////////////////////////////////////////////////
void _Soft_Uart_Print(char *str) {
   while (*str) {  // why not use USART_Print - uses the call stack
      _Soft_Uart_Write(*str++);
//      delay_ms(5);
   }
}

//////////////////////////////////////////////////////////////////////
void _Soft_Uart_Println(char *str) {
   while (*str) {  // why not use USART_Print - uses the call stack
       _Soft_Uart_Write(*str++);
//      delay_ms(5);
   }
   _Soft_Uart_Write('\r');
//      delay_ms(5);
   _Soft_Uart_Write('\n');
//     delay_ms(5);
}



